#include <avr/io.h>
#include <avr/interrupt.h>

#include "hbridge.h"
#include "macros.h"

#define TIMER0_RUN 0b00000011

// g1 = pc0
// g2 = pc1
// g3 = pc2
// g4 = pc3

/*
        ^ VCC
      __|__
   G1 |   | G4
M1 ---|   |--- M2
   G2 |   | G3
      -----
        |
       --- GND
*/

unsigned char portC_on = 0b1111;
unsigned char portC_off = 0b1111;

void timer0_init(void)
{
    set(TIMSK0, TOIE0); // Enable overflow interrupt
    set(TIMSK0, OCIE0B); // Enable output compare A interrupt
    
    // Don't turn on Timer0 - that's done by the motor routines.
}

void hbridge_init(void)
{
    motor_stop();
    DDRC |= 0b1111;
    timer0_init();
}

inline void motor_forward(void)
{
    PORTC = 0b1100;
}

inline void motor_backward(void)
{
    PORTC = 0b0011;
}

inline void motor_run(void)
{
    PORTC = 0b1001;
    TCCR0B = 0; // stop any PWM that might be running
}

inline void motor_stop(void)
{
    PORTC = 0b1111;
    TCCR0B = 0; // stop any PWM that might be running
}

// pwm is between 0 and 255
void motor_forward_pwm(char pwm)
{
    portC_on = 0b1100;
    portC_off = 0b1111;
    OCR0B = pwm;
    TCCR0B = TIMER0_RUN; // Start running
}

// pwm is between 0 and 255
void motor_backward_pwm(char pwm)
{
    portC_on = 0b0011;
    portC_off = 0b1111;
    OCR0B = pwm;
    TCCR0B = TIMER0_RUN; // Start running
}

// Timer0 is used to manually PWM the motors
ISR(TIMER0_OVF_vect) {
    PORTC = portC_on;
}

ISR(TIMER0_COMPB_vect) {
    PORTC = portC_off;
}